/*************************************************************************
 * File Name: NSTableView+MiscEOSorting.m
 * Version  : 0.0 alpha
 * Date     : Wed 23-Jul-1997
 *************************************************************************
 *  COPYWHAT (C) 1997 by Tomi Engel
 *
 * This notice may not be removed from this source code.
 * The use and distribution of this software is governed by the
 * terms of the MiscKit license agreement.  Refer to the license
 * document included with the MiscKit distribution for the terms.
 *                    ALL RIGHTS RESERVED
 *
 *************************************************************************    
 * Notes      : 
 * Bugs       : 
 * Author(s)  : tsengel
 * Last update: $Date: 1997/07/23 17:01:27 $
 * History    : $Log: NSTableView_MiscEOSorting.m,v $
 * History    : Revision 1.1  1997/07/23 17:01:27  tsengel
 * History    : Added new category for easy table sorting
 * History    :
 *************************************************************************/ 

#import <MiscAppKit/NSTableView+MiscColumnHandling.h>
#import "NSTableView+MiscEOSorting.h"

#import <EOControl/EOControl.h>

@implementation NSTableView (MiscEOSorting)

- (NSArray *)currentColumnKeyOrderArray
/*"
    This method returns an array of EOSortOrdering instances build from all available columns using a default "compare" method. It ignores columns with an identifier starting with "_" which are not considered valid keys and there get skipped. Otherwise the identifier is considered to be the key.

    Your table view delegates method should usually look like:

    - (void)tableViewColumnDidMove:(NSNotification *)aNotification
    {
         id		newArray;

         newArray = [documentArray sortedArrayUsingKeyOrderArray:[documentTable currentColumnKeyOrderArrays]];
         [documentArray release];
         documentArray = [newArray retain];
         [documentTable reloadData];
    }
"*/
{
    id	sortArray = [NSMutableArray new];
    id	enumMaster = [self columnEnumerator];
    id	nextColumn;
    id	anIdentifier;

    while( nextColumn = [enumMaster nextObject] )
    {
        anIdentifier = [nextColumn identifier];
        if( [anIdentifier length] > 0 &&
            ![[anIdentifier substringToIndex:1] isEqual:@"_"] )
            [sortArray addObject:[EOSortOrdering sortOrderingWithKey:anIdentifier selector:@selector(compare:)]];
    }

    return [sortArray autorelease];
}


@end
