/*	MiscObjectSkipList.m

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import <Foundation/Foundation.h>

#import "MiscObjectSkipList.h"

@implementation MiscObjectSkipList

- init
{
	self = [super init];
	compareSelector = @selector(compare:);
	compareWithSelector = YES;
	return self;
}

- initCompareSelector:(SEL)comparator
{
	self = [super init];
	compareSelector = comparator;
	compareWithSelector = YES;
	return self;
}

- initCompareFunction:(int(*)(id, id, void *))comparator context:(void *)context
{
	self = [super init];
	compareFunction = comparator;
	compareContext = context;
	compareWithSelector = NO;
	return self;
}

- initWithObjects:(id *)objects count:(unsigned)count
{
	unsigned i;
	
	self = [super init];
	compareSelector = @selector(compare:);
	compareWithSelector = YES;
	for(i = 0;i < count;i++)
		[self insertObject:objects[i]];
	return self;
}

- initWithObjects:(id *)objects count:(unsigned)count compareSelector:(SEL)comparator
{
	unsigned i;
	
	self = [super init];
	compareSelector = comparator;
	compareWithSelector = YES;
	for(i = 0;i < count;i++)
		[self insertObject:objects[i]];
	return self;
}

- initWithObjects:(id *)objects count:(unsigned)count compareFunction:(int(*)(id, id, void *))comparator context:(void *)context
{
	unsigned i;
	
	self = [super init];
	compareFunction = comparator;
	compareContext = context;
	compareWithSelector = NO;
	for(i = 0;i < count;i++)
		[self insertObject:objects[i]];
	return self;
}

- (NSComparisonResult)compareNode:(MiscSkipListNode *)p withObject:anObject
{
	NSComparisonResult result;
	int (*test)(id, SEL, id);
	
	if(!p)
		return NSOrderedDescending;
	if(compareWithSelector){
		test = (int (*)(id, SEL, id))[anObject methodForSelector:compareSelector];
		result = test(p->nodeObject, compareSelector, anObject);
	} else
		result = compareFunction((NSObject *)p->nodeObject, anObject, compareContext);
	return result;
}	

- (void)checkArgument:anObject
{
	if(!anObject)
		[NSException raise:NSInvalidArgumentException format:@"Argument is nil"];
	return;
}

- (void)encodeWithCoder:(NSCoder *)coder 
{
	[super encodeWithCoder:coder];
	[coder encodeValueOfObjCType:@encode(BOOL) at:&compareWithSelector];
	[coder encodeValueOfObjCType:@encode(SEL) at:&compareSelector];
	[coder encodeValueOfObjCType:"?" at:&compareContext];
	[coder encodeValueOfObjCType:"?" at:&compareFunction];
    	[coder encodeObject:[self allObjects]];
}

- initWithCoder:(NSCoder *)coder 
{
	self = [super initWithCoder:coder];
	[coder decodeValueOfObjCType:@encode(BOOL) at:&compareWithSelector];
	[coder decodeValueOfObjCType:@encode(SEL) at:&compareSelector];
	[coder decodeValueOfObjCType:"?" at:&compareContext];
	[coder decodeValueOfObjCType:"?" at:&compareFunction];
	[self insertObjectsFromArray:[coder decodeObject]];
	return self;
}

@end
