#ifndef __MiscTableScrollPrivate_h
#define __MiscTableScrollPrivate_h
//=============================================================================
//
//	Copyright (C) 1995-1997 by Paul S. McCarthy and Eric Sunshine.
//		Written by Paul S. McCarthy and Eric Sunshine.
//			    All Rights Reserved.
//
//	This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the authors
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableScrollPrivate.h
//
//	Methods used within the Table Scroll palette but which are not
//	exported with the *public* interface.
//
// NOTE *1*
//	The OPENSTEP 4.1 (gamma) Objective-C++ compiler neglects to define the
//	macro __PRETTY_FUNCTION__ which is used by the macro NSCAssert() and
//	its cousins, so we have to fake it up.
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScrollPrivate.h,v 1.9 97/04/15 09:09:39 sunshine Exp $
// $Log:	MiscTableScrollPrivate.h,v $
// Revision 1.9  97/04/15  09:09:39  sunshine
// v0.125.8: Added "MiscTableScroll/" prefix to #import to facilitate new
// framework organization.
// 
// Revision 1.8  97/04/09  02:37:25  sunshine
// 0.125.7: 4.1 (gamma) compiler isn't defining __PRETTY_FUNCTION__ which is
// used by NSCAssert() macro and relatives.
// 
// Revision 1.7  97/03/20  17:42:04  sunshine
// v122.1: Added -setTracking: to fix lazy-mode mouse-tracking display
// problem.
//-----------------------------------------------------------------------------
#import <MiscTableScroll/MiscTableScroll.h>
#import "MiscDelegateFlags.h"

#if defined(NX_CURRENT_COMPILER_RELEASE) && NX_CURRENT_COMPILER_RELEASE <= 400
# define __PRETTY_FUNCTION__ "<unknown>"		/* NOTE 1 */
#endif

@interface MiscTableScroll (PrivateInternal)

- (id)responsibleDelegate:(MiscDelegateFlags::Selector)cmd;

// Private: border -> scroll
- (NSString*) border:(MiscBorderType)b getDelegateSlotTitle:(int)slot;
- (id) border:(MiscBorderType)b getDelegateSlotPrototype:(int)s;

// TableScroll(IncrementalSearch) -> TableScroll(Keyboard)
- (void) keyboardSelect:(NSEvent*)p;

- (MiscBorderView*)rowTitles;
- (MiscBorderView*)colTitles;

- (void)setTracking:(BOOL)flag;
- (void)setClickedRow:(MiscCoord_P)r column:(MiscCoord_P)c;
- (void)clearClicked;

- (void)resetSelection; // Reset stale-old-selection in Table & Border views.

@end

#endif // __MiscTableScrollPrivate_h
