//
//	MiscProgressView.h -- a simple view class for displaying progress
//		Written originally by Don Yacktman Copyright (c) 1993 by James Heiser.
//				Modified from an example in the NeXT documentation.
//				This file is maintained by James Heiser, jheiser@adobe.com.
//				Version 1.0.  All rights reserved.
//
//		OPENSTEP conversion and mods:  Todd Anthony Nathan <todd@thoughtful.com>
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>

#define MISC_PROGRESS_DEFAULTSTEPSIZE	5
#define MISC_PROGRESS_MAXSIZE	100

@interface MiscProgressView : NSView
{
	int	total;
	int	count;
	int	stepSize;
	float	ratio;
	NSColor	*bg;	// foreground
	NSColor	*fg;	// background
	NSColor	*bd;	// border colors
}

/*"Initialization and Drawing Support"*/
- initWithFrame:(NSRect)frameRect;
- (void) drawRect:(NSRect)rects;
- (void) dealloc;

/*"Rendering Hook Support"*/
- (void) renderBackground;
- (void) renderBar;
- (void) renderBorder;

/*"Accessor Support"*/
- (int) stepSize;
- (void) setStepSize:(int)value;
- (NSColor *) backgroundColor;
- (void) setBackgroundColor:(NSColor *)color;
- (NSColor *) foregroundColor;
- (void) setForegroundColor:(NSColor *)color;
- (NSColor *) borderColor;
- (void) setBorderColor:(NSColor *)color;
- (void) setRatio:(float)newRatio;

/*"Accessor Support"*/
- (void) takeIntValueFrom:(id)sender;
- (void) increment:(id)sender;
- (void) takeFloatValueFrom:(id)sender;
- (void) clear:(id)sender;

/*"Archiving Support"*/
- (id) initWithCoder:(NSCoder *)aDecoder;
- (void) encodeWithCoder:(NSCoder *)aCoder;

/*"Interface Builder Inspector Support"*/
- (NSString *) inspectorClassName;

@end
