// $Id$

//
// --------------------- MiscTabMatrix Class Interface ------------------------
//
//    MiscTabMatrix is designed to be used with an object that knows how to
// switch its subviews based on the MiscTabMatrix selection.  Set the
// MiscTabMatrix target to the object that knows how to switch its subviews
// using an appropriate action.
//    To use the behavior of a NSMatrix in which the matrix or cell action
// that causes the target's view to switch is sent on the mouse up event or
// when the left or right arrow key is pressed, set useMatrixBehavior to YES.
//    To mimic the behavior of the StackView in IB's Document Window in which
// the switchView's view is switched when the MiscTabMatrix' selectedCell
// changes (mouse down or mouse dragged), set useMatrixBehavior to NO.
//
// Written by Art Isbell (derived from UITabActionCell by Bill Edney,
//     Sean Hill, Mark Onyschuk).
// Copyright 1996 by Art Isbell.
// Version 1.0.  All rights reserved.
//
// This notice may not be removed from this source code.
//
// This object is included in the MiscKit by permission from the author
// and its use is governed by the MiscKit license, found in the file
// "License.rtf" in the MiscKit distribution.  Please refer to that file
// for a list of all applicable permissions and restrictions.
//	
// ----------------------------------------------------------------------------
//

// ----------------------------- Header Files ---------------------------------

#import <AppKit/NSMatrix.h>

// ---------------- Typedef, Struct, and Union Declarations -------------------

// --------------------- Constant and Enum Definitions ------------------------

@interface MiscTabMatrix:NSMatrix
{
// -------------------- Instance Variable Declarations ------------------------
@private
@protected
@public
}
// ---------------------- Factory Method Declarations -------------------------

// ---------------- Overridden Instance Method Declarations -------------------

- (void)highlightCell:(BOOL)aFlag atRow:(int)aRow column:(int)aColumn;

// ------------------- New Instance Method Declarations -----------------------

// ----------------- Delegate Instance Method Declarations --------------------

@end

@protocol MiscTabSwitchViews

- (BOOL)useMatrixBehavior;
- (void)setViewNumber:(int)aViewNumber;

@end
