/*
 *	Filename: Sorting.m
 *	Created : Wed Nov 27 18:34:17 1996
 *	Author  : Vince DeMarco
 *	LastEditDate Was "Mon Feb 17 16:47:01 1997"
 */

#import "Sorting.h"

#import <Foundation/NSString.h>
#import <objc/objc-class.h>
#import <Foundation/NSException.h>
#import <Foundation/NSInvocation.h>
#import <Foundation/NSMethodSignature.h>

#import <strings.h>

typedef struct _ArrayType {
    char               type;
    SEL                aSelector;
    BOOL               ascending;
    NSMethodSignature *signature;
} ArrayType;


#define CMP_RESULT(val1, val2) \
                ((val1) < (val2) ? NSOrderedAscending : ((val1) == (val2) ? NSOrderedSame : NSOrderedDescending))

static int _sort(id num1, id num2, void *context)
{
    ArrayType    *type;
    NSInvocation *anInvocation1;
    NSInvocation *anInvocation2;

    type	  = context;
    anInvocation1 = [NSInvocation invocationWithMethodSignature:type->signature];
    anInvocation2 = [NSInvocation invocationWithMethodSignature:type->signature];

    [anInvocation1 setSelector:type->aSelector];
    [anInvocation1 setTarget:num1];

    [anInvocation2 setSelector:type->aSelector];
    [anInvocation2 setTarget:num2];

    switch(type->type){
    case _C_ID:
	{
	    id result1;
	    id result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return [result1 compare:result2];
	}
    case _C_CHR:
	{
	    char result1;
	    char result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_SHT:
	{
	    short result1;
	    short result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_INT:
	{
	    int result1;
	    int result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_LNG:
	{
	    long int result1;
	    long int result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_FLT:
	{
	    float result1;
	    float result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_DBL:
	{
	    double result1;
	    double result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return CMP_RESULT(result1,result2);
	}
    case _C_CHARPTR:
	{
	    char *result1;
	    char *result2;

	    [anInvocation1 invoke];
	    [anInvocation1 getReturnValue:&result1];

	    [anInvocation2 invoke];
	    [anInvocation2 getReturnValue:&result2];

	    return strcmp(result1,result2);
	}
    default:
	break;
    }
    return 0;
}

@implementation NSArray(NSArray_Sorting)

- (NSComparisonResult)compareObjectAtIndex:(unsigned int)x with:(unsigned int)y usingSelector:(SEL)aSelector
{
    NSMethodSignature *signature;
    ArrayType          type;

    signature = [[self objectAtIndex:x] methodSignatureForSelector:aSelector];
    switch ([signature methodReturnType][0]){
    case _C_ID:
    case _C_CLASS:
	type.type = _C_ID;
	break;
    case _C_CHR:
    case _C_UCHR:
	type.type = _C_CHR;
	break;
    case _C_SHT:
    case _C_USHT:
	type.type = _C_SHT;
	break;
    case _C_INT:
    case _C_UINT:
	type.type = _C_INT;
	break;
    case _C_LNG:
    case _C_ULNG:
	type.type = _C_LNG;
	break;
    case _C_FLT:
	type.type = _C_FLT;
	break;
    case _C_DBL:
	type.type = _C_DBL;
	break;
    case _C_CHARPTR:
	type.type = _C_CHARPTR;
	break;
    default:
	[NSException raise:NSInvalidArgumentException
		    format:@"%@ compareObjectAtIndex: (%d) with:(%d) usingSelector:(%@)",
		     NSStringFromClass([self class]),x,y,NSStringFromSelector(aSelector)];
	return NSOrderedSame;
    }
    type.aSelector = aSelector;
    type.signature = signature;
    return _sort([self objectAtIndex:x], [self objectAtIndex:y],&type);
}


- (NSArray *)sortedArrayUsingReturnOfSelector:(SEL)aSelector
{
    NSMethodSignature *signature;
    ArrayType          type;

    if ([self count] <= 1){
	return self;
    }

    signature = [[self objectAtIndex:0] methodSignatureForSelector:aSelector];

    NSParameterAssert([signature numberOfArguments] == 2);

    switch ([signature methodReturnType][0]){
    case _C_ID:
    case _C_CLASS:
	type.type = _C_ID;
	break;
    case _C_CHR:
    case _C_UCHR:
	type.type = _C_CHR;
	break;
    case _C_SHT:
    case _C_USHT:
	type.type = _C_SHT;
	break;
    case _C_INT:
    case _C_UINT:
	type.type = _C_INT;
	break;
    case _C_LNG:
    case _C_ULNG:
	type.type = _C_LNG;
	break;
    case _C_FLT:
	type.type = _C_FLT;
	break;
    case _C_DBL:
	type.type = _C_DBL;
	break;
    case _C_CHARPTR:
	type.type = _C_CHARPTR;
	break;
    default:
	return nil;
    }
    type.aSelector = aSelector;
    type.signature = signature;
    return [self sortedArrayUsingFunction:_sort context:&type];
}

@end

@implementation NSMutableArray(NSMutableArray_Sorting)

- (void)sortUsingReturnOfSelector:(SEL)aSelector
{
    NSMethodSignature *signature;
    ArrayType          type;

    if ([self count] <= 1){
	return;
    }

    signature = [[self objectAtIndex:0] methodSignatureForSelector:aSelector];

    NSParameterAssert([signature numberOfArguments] == 2);

    switch ([signature methodReturnType][0]){
    case _C_ID:
    case _C_CLASS:
	type.type = _C_ID;
	break;
    case _C_CHR:
    case _C_UCHR:
	type.type = _C_CHR;
	break;
    case _C_SHT:
    case _C_USHT:
	type.type = _C_SHT;
	break;
    case _C_INT:
    case _C_UINT:
	type.type = _C_INT;
	break;
    case _C_LNG:
    case _C_ULNG:
	type.type = _C_LNG;
	break;
    case _C_FLT:
	type.type = _C_FLT;
	break;
    case _C_DBL:
	type.type = _C_DBL;
	break;
    case _C_CHARPTR:
	type.type = _C_CHARPTR;
	break;
    default:
	return;
    }
    type.aSelector = aSelector;
    type.signature = signature;
    [self sortUsingFunction:_sort context:&type];
}

@end
