/*
 *	Filename: RegExFormatter.h
 *	Created : Mon Jan 20 12:48:50 1997
 *	Author  : Vince DeMarco
 *	LastEditDate Was "Tue Jan 21 18:03:45 1997"
 */

#import <Foundation/NSFormatter.h>

@class NSArray,NSMutableArray,NSString;

@class RegEx;
@interface RegExFormatter : NSFormatter <NSCopying, NSCoding>
{
@private
    NSMutableArray *_expressions;
    NSString       *_formatPattern;
    RegEx          *_lastMatchedExpression;
    BOOL            _allowsEmptyString;
}

- (id)initWithContentsOfFile:(NSString *)path;

/* Designated initializer */
- (id)initWithRegularExpressions:(NSArray *)expressions;
- (id)initWithRegularExpression:(RegEx *)expression;

- (void)addRegularExpression:(RegEx *)expression;
- (void)removeRegularExpressionAtIndex:(unsigned)index;
- (RegEx *)regularExpressionAtIndex:(unsigned)index;

- (NSArray *)expressions;

- (BOOL)allowsEmptyString;
- (void)setAllowsEmptyString:(BOOL)flag;

- (NSString *)formatPattern;
- (void)setFormatPattern:(NSString *)pattern;

- (RegEx *)lastMatchedExpression;


/* Return a formated "anObject" based on the format given in the formatPattern */
- (NSString *)stringForObjectValue:(id)anObject;

/* The value returned in anObject by this method will be the equivalent to if the format Pattern
 * was set to %0
 */
- (BOOL)getObjectValue:(id *)anObject forString:(NSString *)string errorDescription:(NSString **)error;

@end

/* Keys used when reading the configuration from a dictionary file
 * loaded from disk
 */
extern NSString *RegExFormatterRegex;       /* Key to array of Regex patterns */
extern NSString *RegExFormatterFormatter;   /* Key to the FormatPattern */
extern NSString *RegExFormatterAllowEmpty;  /* Key to the Allows Empty String */

